
local Class = require"libs.middleclass_320"
local Stateful = require"libs.stateful_103"

local Spielfeld = Class("Spielfeld"):include(Stateful)



 function Spielfeld:initialize(params)
	
	self.size = params.numOfFields -- how many fields in x/y direction (map is square)
	self.fieldSizePx = params.fieldSizePx -- size of a single field in pixels
	self.mapSizePx = self.size * self.fieldSizePx -- edge length of whole map in pixels
	
	self.map = {} -- hashtable, key: coordstring (eg. "x04y09"), value: {size, colorindex)
	self.mapByContent = {}
	for i = 0, 2 do self.mapByContent[i] = {} end
	
	self.fieldCanvas = love.graphics.newCanvas(self.fieldSizePx, self.fieldSizePx)
	self.canvas = love.graphics.newCanvas(self.fieldSizePx * self.size, self.fieldSizePx * self.size)
	
	for x = 1, self.size do
		for y = 1, self.size do
			self:setPos{x = x, y = y, size = 0, col = love.math.random(Spielfeld.static.maxColors)}
		end
	end
	
	for i = 1, 5 do
		local emptySpot = love.math.random(#self.mapByContent[0])
		self:setPos{coordStr = self.mapByContent[0][emptySpot], size = 1, col = love.math.random(Spielfeld.static.maxColors)}
	end
	
	for i = 0, self.size + 1 do -- fill invisible borders around map with fake balls in fake color to stop floodfill
		self:setPos{x = 0, y = i, size = 2, col = 999}
		self:setPos{x = self.size + 1, y = i, size = 2, col = 999}
		self:setPos{x = i, y = 0, size = 2, col = 999}
		self:setPos{x = i, y = self.size + 1, size = 2, col = 999}
	end
	
	self.floodfillColorindex = 0
	
end



--[[	pt{ x = x, y = y, coordStr = "x##y##" -- either x/y or coordStr,
					size = #(0-2), col = #(index), ffcol = #(index), highl = bool,
					special = "none"|"bomb"|etc, displScale = #(0.0 - 1.0)
--]]
function Spielfeld:setPos(pt) -- parameter table, see above
	
	local coordStr
	if pt.x then coordStr = Spielfeld.static.getCoordString(pt.x, pt.y) end
	if pt.coordStr then coordStr = pt.coordStr end
	local oldSize = 0
	if self.map[coordStr] then oldSize = self.map[coordStr].size end
	
	local removeIndex = Spielfeld.static.findInArray( self.mapByContent[oldSize], coordStr)
	if removeIndex then table.remove( self.mapByContent[oldSize], removeIndex ) end
	
	local ffcol = pt.ffcol or 0
	if pt.size == 2 then ffcol = -1 end
	
	pt.special = pt.special or "none"
	pt.displScale = pt.displScale or 1
	
	self.map[coordStr] = {size = pt.size, col = pt.col, ffcol = ffcol,
		highl = pt.highl, special = pt.special, displScale = pt.displScale}
	
	table.insert(self.mapByContent[pt.size], coordStr)
	
end



function Spielfeld:clearFFcol()
	
	for x = 1, self.size do
		for y = 1, self.size do
			local coordStr = Spielfeld.static.getCoordString(x, y)
			local newffcol = 0
			if self.map[coordStr].size == 2 then newffcol = -1 end
			self.map[coordStr].ffcol = newffcol
		end
	end
	
end



function Spielfeld:draw(xOffs, yOffs)
	
	love.graphics.setCanvas(self.canvas)
	love.graphics.rectangle("line", 0, 0, self.size * self.fieldSizePx, self.size * self.fieldSizePx)
	for x = 1, self.size do
		for y = 1, self.size do
			love.graphics.draw(self:drawField(x, y), (x-1) * self.fieldSizePx, (y-1) * self.fieldSizePx)
		end
	end
	love.graphics.setCanvas()
	love.graphics.draw(self.canvas, xOffs, yOffs)
end



function Spielfeld:drawField(x, y)
	
	local coordStr = Spielfeld.static.getCoordString(x, y)
	
	local size = self.map[coordStr].size
	local colTbl = Spielfeld.static.colors[ self.map[coordStr].col ]
	local ffcol = self.map[coordStr].ffcol
	local ffcolTbl = Spielfeld.static.colors[ self.map[coordStr].ffcol ]
	local highl = self.map[coordStr].highl
	local special = self.map[coordStr].special
	
	
	love.graphics.setCanvas(self.fieldCanvas)
	love.graphics.clear(0, 0, 0)
	
	if highl then
		love.graphics.setColor(colTbl)
		love.graphics.setLineWidth(4)
		love.graphics.rectangle("line", 0, 0, self.fieldSizePx, self.fieldSizePx, 8)
	else
		love.graphics.rectangle("line", 0, 0, self.fieldSizePx, self.fieldSizePx)
	end
	
	if ffcol > 0 then
		local r, g, b = ffcolTbl[1], ffcolTbl[2], ffcolTbl[3]
		love.graphics.setColor(r, g, b, 128)
		love.graphics.setLineWidth(2)
		love.graphics.rectangle("line", 4, 4, self.fieldSizePx - 8, self.fieldSizePx - 8)
		love.graphics.setColor(r, g, b, 64)
		love.graphics.rectangle("fill", 1, 1, self.fieldSizePx - 2, self.fieldSizePx - 2)
		love.graphics.setLineWidth(1)
	end

	if size == 1 then
		love.graphics.setColor(colTbl)
		love.graphics.circle("fill", self.fieldSizePx/2, self.fieldSizePx/2, self.fieldSizePx/8, 12)
		
	elseif size == 2 then
		love.graphics.setColor(colTbl)
		love.graphics.circle("fill", self.fieldSizePx/2, self.fieldSizePx/2, self.fieldSizePx/2.5, 32)
		love.graphics.setBlendMode("add")
		love.graphics.setColor(15, 15, 15)
		love.graphics.circle("fill", self.fieldSizePx/2.2, self.fieldSizePx/2.2, self.fieldSizePx/3, 32)
		love.graphics.circle("fill", self.fieldSizePx/2.3, self.fieldSizePx/2.3, self.fieldSizePx/4, 32)
		love.graphics.circle("fill", self.fieldSizePx/2.4, self.fieldSizePx/2.4, self.fieldSizePx/5, 32)
		love.graphics.circle("fill", self.fieldSizePx/2.4, self.fieldSizePx/2.4, self.fieldSizePx/6, 32)
		love.graphics.setBlendMode("alpha")
			
		if highl then
			love.graphics.setBlendMode("add")
			love.graphics.setColor(40, 40, 40)
			love.graphics.circle("fill", self.fieldSizePx/2.2, self.fieldSizePx/2.2, self.fieldSizePx/3, 32)
			love.graphics.circle("fill", self.fieldSizePx/2.3, self.fieldSizePx/2.3, self.fieldSizePx/4, 32)
			love.graphics.circle("fill", self.fieldSizePx/2.4, self.fieldSizePx/2.4, self.fieldSizePx/5, 32)
			love.graphics.circle("fill", self.fieldSizePx/2.4, self.fieldSizePx/2.4, self.fieldSizePx/6, 32)
			love.graphics.setBlendMode("alpha")
		end

		if special then
			love.graphics.setColor(0,0,0)
			local fs = self.fieldSizePx
			
			if special == "bomb" then
				love.graphics.polygon("fill", fs/2, fs/2, fs/2-2, fs/2+2, fs*0.3, fs*0.2)
				love.graphics.polygon("fill", fs/2-1, fs/2, fs/2+1, fs/2, fs*0.45, fs*0.1)
				love.graphics.polygon("fill", fs/2-1, fs/2, fs/2+1, fs/2, fs*0.45, fs*0.8)
				love.graphics.polygon("fill", fs/2, fs/2, fs/2-2, fs/2+2, fs*0.75, fs*0.8)
				love.graphics.polygon("fill", fs/2, fs/2, fs/2, fs/2+3, fs*0.75, fs*0.5)
				love.graphics.polygon("fill", fs/2, fs/2, fs/2, fs/2+3, fs*0.25, fs*0.55)
				
			elseif special == "locked" then
				love.graphics.line(fs*0.4, fs*0.4, fs*0.6, fs*0.6)
				love.graphics.circle("line", fs/2, fs/2, fs*0.2, 12)
				
			elseif special == "recolor" then
				local points = { {0.4, 0.6} , {0.5, 0.5}, {0.6, 0.4}, {0.4, 0.45}, {0.65, 0.55} }
				for i, v in ipairs(points) do
					love.graphics.setColor(Spielfeld.static.colors[i])
					love.graphics.circle("fill", fs*v[1], fs*v[2], fs*(0.2 - i/40), 8)
				end
			end
		end
	end

		
	love.graphics.setCanvas(self.canvas)
	love.graphics.setBlendMode("alpha")
	love.graphics.setLineWidth(1)
	love.graphics.setColor{255, 255, 255}
	
	return self.fieldCanvas

end



Spielfeld.static.colors = {
	{200, 50, 50},
	{50, 200, 50},
	{50, 50, 200},
	{200, 200, 50},
	{200, 50, 200}
}
Spielfeld.static.maxColors = #Spielfeld.static.colors


Spielfeld.static.specials = { "bomb", "locked", "recolor" }


function Spielfeld.static.getCoordString(x, y)
	return table.concat({"x", string.format("%02d", x), "y", string.format("%02d", y)})
end
function Spielfeld.static.splitCoordString(s)
	local x, y = string.sub(s, 2, 3), string.sub(s, 5, 6)
	return tonumber(x), tonumber(y)
end


function Spielfeld.static.findInArray(array, value)
	
	local length, i, v = #array, 1, array[1]
	while value ~= v and i < length do
		i = i + 1
		v = array[i]
	end
	if v == value then return i else return nil end
	
end


return Spielfeld